@@ -5,6 +5,7 @@
 module PMS.Infra.ProcSpawn.DS.Core where
 
 import System.IO
+import System.Environment 
 import Control.Monad.Logger
 import Control.Monad.IO.Class
 import Control.Monad.Trans.Class
@@ -21,6 +22,7 @@ import System.Exit
 import qualified Data.Text.Encoding as TE
 import qualified Data.ByteString as BS
 import Data.Aeson 
+import Data.List
 import qualified System.Process as S
 import qualified Data.ByteString.Char8 as BS8
 import qualified Data.ByteString.Lazy.Char8 as BL
@@ -158,12 +160,14 @@ genProcSpawn cmdDat = do
 
   prompts   <- view DM.promptsDomainData <$> lift ask
   resQ      <- view DM.responseQueueDomainData <$> lift ask
+  invalidChars <- view DM.invalidCharsDomainData <$> lift ask
+  invalidCmds <- view DM.invalidCmdsDomainData <$> lift ask
   procMVar  <- view processAppData <$> ask
   lockTMVar <- view lockAppData <$> ask
 
   (cmdTmp, argsArrayTmp, addPrompts)  <- getCommandArgs name argsBS
-  cmd <- liftIOE $ DM.validateCommand cmdTmp
-  argsArray <- liftIOE $ DM.validateArgs argsArrayTmp
+  cmd <- liftIOE $ DM.validateCommand invalidChars invalidCmds cmdTmp
+  argsArray <- liftIOE $ DM.validateArgs invalidChars argsArrayTmp
 
   $logDebugS DM._LOGTAG $ T.pack $ "genProcRunTask: cmd. " ++ cmd ++ " " ++ show argsArray
  
@@ -228,7 +232,7 @@ procSpawnTask cmdDat resQ procVar lockTMVar cmd args addEnv prompts tout = do
   STM.atomically (STM.takeTMVar procVar) >>= \case
     Just p -> do
       STM.atomically $ STM.putTMVar procVar $ Just p
-      hPutStrLn stderr "[ERROR] PMS.Infrastructure.DS.Core.procSpawnTask: pms is already connected."
+      hPutStrLn stderr "[ERROR] PMS.Infrastructure.DS.Core.procSpawnTask: process is already running."
       toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "process is already running."
     Nothing -> do
       flip E.catchAny errHdl $ runProc procVar cmd args addEnv
@@ -283,13 +287,13 @@ genProcCMDTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errH
     Just p -> do
       STM.atomically $ STM.putTMVar procVar $ Just p
       hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is already running."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is already running."
-    Nothing -> runProc procVar "cmd" [] []
+      toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "process is already running."
+    Nothing -> flip E.catchAny errHdl $ runProc procVar "cmd" [] []
 
   STM.atomically (STM.readTMVar procVar) >>= \case
     Nothing -> do
       hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is not started."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcCMDTask: process is not started."
+      toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "process is not started."
     Just procDat -> do
       let wHdl = procDat^.wHdLProcData
           msg  =  "chcp 65001 & prompt $P$G$G$G"
@@ -344,13 +348,13 @@ genProcPSTask :: DM.ProcRunCommandData
 genProcPSTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errHdl $ do
   hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask start. "
   let initCmd = "chcp 65001; $OutputEncoding = [Console]::OutputEncoding = New-Object System.Text.UTF8Encoding $false; function prompt { \"PS $((Get-Location).Path)>>>\" }"
-
+  env <- addPathsToEnv ["C:\\Windows\\System32"]
   STM.atomically (STM.takeTMVar procVar) >>= \case
     Just p -> do
       STM.atomically $ STM.putTMVar procVar $ Just p
       hPutStrLn stderr "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask: process is already running."
-      E.throwString "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask: process is already running."
-    Nothing -> runProc procVar "powershell" ["-NoLogo", "-NoExit", "-Command", initCmd] []
+      toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" "process is already running."
+    Nothing -> runProc procVar "powershell" ["-NoLogo", "-NoExit", "-Command", initCmd] env
 
   STM.atomically (STM.readTMVar procVar) >>= \case
     Just p -> race (DM.expect lockTMVar (readProc p) prompts) (CC.threadDelay tout) >>= \case
@@ -369,6 +373,14 @@ genProcPSTask cmdDat resQ procVar lockTMVar prompts tout = flip E.catchAny errHd
       hPutStrLn stderr $ "[ERROR] PMS.Infra.ProcSpawn.DS.Core.genProcPSTask: exception occurred. " ++ show e
       toolsCallResponse resQ (cmdDat^.DM.jsonrpcProcRunCommandData) (ExitFailure 1) "" (show e)
 
+    addPathsToEnv :: [FilePath] -> IO [(String, String)]
+    addPathsToEnv extraPaths = do
+        env <- getEnvironment
+        let sep = ";"  -- Windows用。Unix系なら ":"
+            oldPath = maybe "" id (lookup "PATH" env)
+            newPath = intercalate sep extraPaths ++ (if null oldPath then "" else sep ++ oldPath)
+        return $ ("PATH", newPath) : filter ((/= "PATH") . fst) env
+
 -- |
 --
 genProcTerminateTask :: DM.ProcTerminateCommandData -> AppContext (IOTask ())
@@ -416,13 +428,15 @@ genProcMessageTask cmdData = do
       tout = 30 * 1000 * 1000
   prompts <- view DM.promptsDomainData <$> lift ask
   resQ <- view DM.responseQueueDomainData <$> lift ask
+  invalidChars <- view DM.invalidCharsDomainData <$> lift ask
+  invalidCmds <- view DM.invalidCmdsDomainData <$> lift ask
   procTMVar <- view processAppData <$> ask
   lockTMVar <- view lockAppData <$> ask
   argsDat <- liftEither $ eitherDecode $ argsBS
   let args = argsDat^.argumentsProcStringToolParams
 
   $logDebugS DM._LOGTAG $ T.pack $ "genProcMessageTask: args. " ++ args
-  return $ procMessageTask cmdData resQ procTMVar lockTMVar args prompts tout
+  return $ procMessageTask cmdData resQ procTMVar lockTMVar args prompts tout invalidChars invalidCmds
 
 -- |
 --
@@ -433,8 +447,10 @@ procMessageTask :: DM.ProcMessageCommandData
                 -> String  -- arguments line
                 -> [String]  -- prompt list
                 -> Int       -- timeout microsec
+                -> [String]  -- invalidChars
+                -> [String]  -- invalidCmds
                 -> IOTask ()
-procMessageTask cmdDat resQ procTMVar lockTMVar args prompts tout = flip E.catchAny errHdl $ do
+procMessageTask cmdDat resQ procTMVar lockTMVar args prompts tout invalidChars invalidCmds = flip E.catchAny errHdl $ do
   hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.procMessageTask run. " ++ args
     
   STM.atomically (STM.readTMVar procTMVar) >>= \case
@@ -456,7 +472,7 @@ procMessageTask cmdDat resQ procTMVar lockTMVar args prompts tout = flip E.catch
     go pDat = do
       let wHdl = pDat^.wHdLProcData
 
-      msg <- DM.validateMessage args
+      msg <- DM.validateMessage invalidChars invalidCmds args
       let cmd = TE.encodeUtf8 $ T.pack $ msg ++ DM._LF
 
       hPutStrLn stderr $ "[INFO] PMS.Infra.ProcSpawn.DS.Core.procMessageTask writeProc : " ++ BS8.unpack cmd
